@@ -500,7 +500,8 @@ class Univariate:
         print("\nGeometric Mean: \n  The geometric mean indicates the central tendency or typical value of a set of "
               "numbers by using the product of their values. The geometric mean applies only to positive numbers. the "
               "geometric mean is only correct mean when averaging normalized results; that is, results that are "
-              "presented as ratios to reference values. The geometric mean of this variable is: ", self.geometric_mean())
+              "presented as ratios to reference values. The geometric mean of this variable is: ",
+              self.geometric_mean())
 
         print("\nHarmonic Mean: \n The harmonic mean can be expressed as the reciprocal of the arithmetic mean of "
               "the reciprocals of the given set of observations. For all positive data sets containing at least one "
@@ -562,7 +563,7 @@ class Univariate:
 
         print("\nThe standard deviation is the square root of the variance. This variable has a standard deviation of ",
               self.standard_deviation(),
-              "It represents a typical deviation from the mean. It is a popular measure of variability because it" 
+              "It represents a typical deviation from the mean. It is a popular measure of variability because it"
               "returns to the original units of measure of the data set.")
         self.probability_density_function()
         plt.show()
@@ -589,6 +590,7 @@ class Univariate:
         plt.show()
         self.violin_plot()
 
+
 class Bivariate:
     def __init__(self, variable_1, variable_2):
         self.data_1 = variable_1
@@ -609,3 +611,155 @@ class Bivariate:
 
 class Multivariate:
     pass
+
+
+class TimeSeries:
+
+    def __init__(self, data, variable_name):
+        self.data = data
+        self.variable_name = variable_name
+
+    def baseplot(self, y_label_):
+        # Set figure size and axis
+        fig, ax = plt.subplots(figsize=(20, 10))
+
+        # Specify data source
+        ax = sns.lineplot(x='Date', y=self.variable_name, data=self.data, color='black')
+
+        # Specify size of graph fonts
+        for item in ([ax.xaxis.label, ax.yaxis.label] +
+                     ax.get_xticklabels() + ax.get_yticklabels()):
+            item.set_fontsize(20)
+        ax.title.set_fontsize(25)
+
+        # Specify Changes to y axis title.
+        plt.ylabel(y_label_)
+
+        # Fill graph below plot line
+        plt.fill_between(self.data['Date'], self.data[self.variable_name], alpha=0.2)
+
+    def simple_moving_averages(self, periods=[10, 25, 50, 200]):
+        for i in periods:
+            col_name = str(i) + '_sma'
+            self.data[col_name] = self.data[self.variable_name].rolling(i).mean()
+
+    def simple_moving_average_plot(self, y_label_):
+        # Plot the base
+        self.baseplot(y_label_=y_label_)
+
+        simple_moving_averages = list(self.data.columns[2:6])
+
+        # Plot averages
+        for i in simple_moving_averages:
+            # Plot all simple moving averages
+            sns.lineplot(x='Date', y=i, data=self.data)
+
+        # View Plot
+        plt.legend(simple_moving_averages)
+        plt.ylabel(y_label_)
+
+    def simple_moving_average_subplots(self):
+        sub_plot_list = list(self.data.columns[1:6])
+
+        fig, axs = plt.subplots(len(sub_plot_list), figsize=(20, 10), sharex=True)
+        fig.suptitle('Simple Moving Average Subplots\n', fontsize=25)
+
+        for i, index in enumerate(sub_plot_list):
+            axs[i].plot(self.data['Date'], self.data[self.variable_name])
+            axs[i].plot(self.data['Date'], self.data[index])
+            axs[i].fill_between(self.data['Date'], self.data[self.variable_name], alpha=0.2)
+            axs[i].title.set_text('\n' + index)
+            for item in ([axs[i].xaxis.label, axs[i].yaxis.label] +
+                         axs[i].get_xticklabels() + axs[i].get_yticklabels()):
+                item.set_fontsize(15)
+            axs[i].title.set_fontsize(20)
+            plt.subplots_adjust(bottom=-0.7)
+
+    def year_on_year_growth(self):
+        self.data['Shift_1'] = self.data[self.variable_name].shift(1)
+        self.data['YoY Change'] = self.data[self.variable_name] - self.data['Shift_1']
+
+    def year_on_year(self, y_label_):
+        # Set figure size and axis
+        fig, ax = plt.subplots(figsize=(20, 10))
+
+        # Set plot title
+        plt.title('Year on Year Change\n')
+
+        # Specify data source
+        ax = sns.lineplot(x='Date', y='YoY Change', data=self.data, color='black')
+
+        # Specify size of graph fonts
+        for item in ([ax.xaxis.label, ax.yaxis.label] +
+                     ax.get_xticklabels() + ax.get_yticklabels()):
+            item.set_fontsize(20)
+        ax.title.set_fontsize(25)
+
+        # Specify Changes to y axis title.
+        plt.ylabel(y_label_)
+
+        # Fill graph below plot line
+        plt.fill_between(self.data['Date'], self.data['YoY Change'], alpha=0.2)
+
+    def adf_test(self, title=''):
+        """
+        Pass in a time series and an optional title, returns an ADF report
+        """
+        print('Augmented Dickey-Fuller Test: {}'.format(title))
+        result = adfuller(self.data[self.variable_name].dropna(), autolag='AIC')  # .dropna() handles differenced data
+
+        labels = ['ADF test statistic', 'p-value', '# lags used', '# observations']
+        out = pd.Series(result[0:4], index=labels)
+
+        for key, val in result[4].items():
+            out['critical value ({})'.format(key)] = val
+
+        print(out.to_string())  # .to_string() removes the line "dtype: float64"
+
+        if result[1] <= 0.05:
+            print("Strong evidence against the null hypothesis")
+            print("Reject the null hypothesis")
+            print("Data has no unit root and is stationary")
+        else:
+            print("Weak evidence against the null hypothesis")
+            print("Fail to reject the null hypothesis")
+            print("Data has a unit root and is non-stationary")
+
+    def difference_plot(self):
+        result = adfuller(self.data[self.variable_name].dropna(), autolag='AIC')
+        fig, ax = plt.subplots(nrows=2, ncols=2, figsize=(15, 11))
+
+        self.data['Diff 1'] = diff(self.data[self.variable_name], k_diff=1)
+        self.data['Diff 2'] = diff(self.data[self.variable_name], k_diff=2)
+        self.data['Diff 3'] = diff(self.data[self.variable_name], k_diff=3)
+
+        self.data[self.variable_name].plot(title="Initial Data", ax=ax[0][0]).autoscale(axis='x', tight=True);
+        self.data['Diff 1'].plot(title="First Difference Data", ax=ax[0][1]).autoscale(axis='x', tight=True);
+        self.data['Diff 2'].plot(title="Second Difference Data", ax=ax[1][0]).autoscale(axis='x', tight=True);
+        self.data['Diff 3'].plot(title="Third Difference Data", ax=ax[1][1]).autoscale(axis='x', tight=True);
+
+    def auto_correlation_plot(self, no_lags=60):
+        fig, ax = plt.subplots(nrows=1, ncols=2, figsize=(20, 10))
+        autocorr = acf(self.data[self.variable_name], nlags=no_lags)
+
+        plot_acf(self.data[self.variable_name].tolist(), lags=no_lags, ax=ax[0]);  # just the plot
+        plot_pacf(self.data[self.variable_name].tolist(), lags=no_lags, ax=ax[1]);  # just the plot
+
+    def lag_plot(self, y_label_):
+        # Set figure size and axis
+        fig, ax = plt.subplots(figsize=(20, 10))
+
+        # Set plot title
+        plt.title(y_label_)
+
+        # Specify data source
+        ax = sns.scatterplot(x=self.variable_name, y='Shift_1', data=self.data)
+
+        # Specify size of graph fonts
+        for item in ([ax.xaxis.label, ax.yaxis.label] +
+                     ax.get_xticklabels() + ax.get_yticklabels()):
+            item.set_fontsize(20)
+        ax.title.set_fontsize(25)
+
+        # Specify Changes to y axis title.
+        plt.ylabel(y_label_)
